@@ -6,7 +6,7 @@
 objects imported this way starts with ``i`` to minimize collisions.
 
 ``ipipe`` supports "pipeline expressions", which is something resembling Unix
-pipes. An example is:
+pipes. An example is::
 
     >>> ienv | isort("key.lower()")
 
@@ -45,8 +45,9 @@ three extensions points (all of them optional):
 
 * Objects that can be iterated by ``Pipe``s must iterable. For special cases,
   where iteration for display is different than the normal iteration a special
-  implementation can be registered with the generic function ``xiter``. This makes
-  it possible to use dictionaries and modules in pipeline expressions, for example:
+  implementation can be registered with the generic function ``xiter``. This
+  makes it possible to use dictionaries and modules in pipeline expressions,
+  for example::
 
       >>> import sys
       >>> sys | ifilter("isinstance(value, int)") | idump
@@ -61,16 +62,16 @@ three extensions points (all of them optional):
 
   Note: The expression strings passed to ``ifilter()`` and ``isort()`` can
   refer to the object to be filtered or sorted via the variable ``_`` and to any
-  of the attributes of the object, i.e.:
+  of the attributes of the object, i.e.::
 
       >>> sys.modules | ifilter("_.value is not None") | isort("_.key.lower()")
 
-  does the same as
+  does the same as::
 
       >>> sys.modules | ifilter("value is not None") | isort("key.lower()")
 
   In addition to expression strings, it's possible to pass callables (taking
-  the object as an argument) to ``ifilter()``, ``isort()`` and ``ieval()``:
+  the object as an argument) to ``ifilter()``, ``isort()`` and ``ieval()``::
 
       >>> sys | ifilter(lambda _:isinstance(_.value, int)) \
       ...     | ieval(lambda _: (_.key, hex(_.value))) | idump
@@ -82,8 +83,10 @@ three extensions points (all of them optional):
       maxunicode |0xffff
 """
 
+skip_doctest = True  # ignore top-level docstring as a doctest.
+
 import sys, os, os.path, stat, glob, new, csv, datetime, types
-import itertools, mimetypes
+import itertools, mimetypes, StringIO
 
 try: # Python 2.3 compatibility
     import collections
@@ -111,6 +114,11 @@ except NameError:
             items.reverse()
         return items
 
+try: # Python 2.4 compatibility
+    GeneratorExit
+except NameError:
+    GeneratorExit = SystemExit
+
 try:
     import pwd
 except ImportError:
@@ -122,21 +130,21 @@ except ImportError:
     grp = None
 
 from IPython.external import simplegeneric
+from IPython.external import path
 
-import path
 try:
-    from IPython import genutils, ipapi
+    from IPython import genutils, generics
 except ImportError:
     genutils = None
-    ipapi = None
+    generics = None
 
-import astyle
+from IPython import ipapi
 
 
 __all__ = [
     "ifile", "ils", "iglob", "iwalk", "ipwdentry", "ipwd", "igrpentry", "igrp",
-    "icsv", "ix", "ichain", "isort", "ifilter", "ieval", "ienum", "ienv",
-    "idump", "iless"
+    "icsv", "ix", "ichain", "isort", "ifilter", "ieval", "ienum",
+    "ienv", "ihist", "ialias", "icap", "idump", "iless"
 ]
 
 
@@ -638,14 +646,18 @@ def xrepr(item, mode="default"):
     There are four different possible values for ``mode`` depending on where
     the ``Display`` object will display ``item``:
 
-    * ``"header"``: ``item`` will be displayed in a header line (this is used by
-      ``ibrowse``).
-    * ``"footer"``: ``item`` will be displayed in a footer line (this is used by
-      ``ibrowse``).
-    * ``"cell"``: ``item`` will be displayed in a table cell/list.
-    * ``"default"``: default mode. If an ``xrepr`` implementation recursively
-      outputs objects, ``"default"`` must be passed in the recursive calls to
-      ``xrepr``.
+    ``"header"``
+        ``item`` will be displayed in a header line (this is used by ``ibrowse``).
+
+    ``"footer"``
+        ``item`` will be displayed in a footer line (this is used by ``ibrowse``).
+
+    ``"cell"``
+        ``item`` will be displayed in a table cell/list.
+
+    ``"default"``
+        default mode. If an ``xrepr`` implementation recursively outputs objects,
+        ``"default"`` must be passed in the recursive calls to ``xrepr``.
 
     If no implementation is registered for ``item``, ``xrepr`` will try the
     ``__xrepr__`` method on ``item``. If ``item`` doesn't have an ``__xrepr__``
@@ -659,7 +671,7 @@ def xrepr(item, mode="default"):
         try:
             for x in func(mode):
                 yield x
-        except (KeyboardInterrupt, SystemExit):
+        except (KeyboardInterrupt, SystemExit, GeneratorExit):
             raise
         except Exception:
             yield (astyle.style_default, repr(item))
@@ -833,20 +845,20 @@ def upgradexattr(attr):
     """
     Convert an attribute descriptor string to a real descriptor object.
 
-    If attr already is a descriptor object return if unmodified. A
+    If attr already is a descriptor object return it unmodified. A
     ``SelfDescriptor`` will be returned if ``attr`` is ``None``. ``"foo"``
     returns an ``AttributeDescriptor`` for the attribute named ``"foo"``.
     ``"foo()"`` returns a ``MethodDescriptor`` for the method named ``"foo"``.
     ``"-foo"`` will return an ``IterAttributeDescriptor`` for the attribute
     named ``"foo"`` and ``"-foo()"`` will return an ``IterMethodDescriptor``
-    for the method named ``"foo"``. Furthermore integer will return the appropriate
+    for the method named ``"foo"``. Furthermore integers will return the appropriate
     ``IndexDescriptor`` and callables will return a ``FunctionDescriptor``.
     """
     if attr is None:
         return selfdescriptor
     elif isinstance(attr, Descriptor):
         return attr
-    elif isinstance(attr, str):
+    elif isinstance(attr, basestring):
         if attr.endswith("()"):
             if attr.startswith("-"):
                 return IterMethodDescriptor(attr[1:-2])
@@ -873,19 +885,21 @@ def xattrs(item, mode="default"):
 
     There are two possible modes:
 
-    * ``"detail"``: The ``Display`` object wants to display a detailed list
-      of the object attributes.
-    * ``"default"``: The ``Display`` object wants to display the object in a
-      list view.
+    ``"detail"``
+        The ``Display`` object wants to display a detailed list of the object
+        attributes.
+
+    ``"default"``
+        The ``Display`` object wants to display the object in a list view.
 
     If no implementation is registered for the object ``item`` ``xattrs`` falls
     back to trying the ``__xattrs__`` method of the object. If this doesn't
     exist either, ``dir(item)`` is used for ``"detail"`` mode and ``(None,)``
     for ``"default"`` mode.
 
-    The implementation must yield attribute descriptor (see the class
+    The implementation must yield attribute descriptors (see the class
     ``Descriptor`` for more info). The ``__xattrs__`` method may also return
-    attribute descriptor string (and ``None``) which will be converted to real
+    attribute descriptor strings (and ``None``) which will be converted to real
     descriptors by ``upgradexattr()``.
     """
     try:
@@ -1180,7 +1194,7 @@ def xrepr_ifile(self, mode="default"):
     except IOError:
         name = "ifile"
         style = astyle.style_default
-    if mode == "cell" or mode in "header" or mode == "footer":
+    if mode in ("cell", "header", "footer"):
         abspath = repr(path._base(self.normpath()))
         if abspath.startswith("u"):
             abspath = abspath[2:-1]
@@ -1199,11 +1213,15 @@ class ils(Table):
     """
     List the current (or a specified) directory.
 
-    Examples:
+    Examples::
 
         >>> ils
+        <class 'IPython.Extensions.ipipe.ils'>
         >>> ils("/usr/local/lib/python2.4")
+        IPython.Extensions.ipipe.ils('/usr/local/lib/python2.4')
         >>> ils("~")
+        IPython.Extensions.ipipe.ils('/home/fperez')
+        # all-random
     """
     def __init__(self, base=os.curdir, dirs=True, files=True):
         self.base = os.path.expanduser(base)
@@ -1237,9 +1255,10 @@ class iglob(Table):
     List all files and directories matching a specified pattern.
     (See ``glob.glob()`` for more info.).
 
-    Examples:
+    Examples::
 
         >>> iglob("*.py")
+        IPython.Extensions.ipipe.iglob('*.py')
     """
     def __init__(self, glob):
         self.glob = glob
@@ -1262,11 +1281,15 @@ class iglob(Table):
 
 class iwalk(Table):
     """
-    List all files and directories in a directory and it's subdirectory.
+    List all files and directories in a directory and it's subdirectory::
 
         >>> iwalk
-        >>> iwalk("/usr/local/lib/python2.4")
+        <class 'IPython.Extensions.ipipe.iwalk'>
+        >>> iwalk("/usr/lib")
+        IPython.Extensions.ipipe.iwalk('/usr/lib')
         >>> iwalk("~")
+        IPython.Extensions.ipipe.iwalk('/home/fperez')  # random
+        
     """
     def __init__(self, base=os.curdir, dirs=True, files=True):
         self.base = os.path.expanduser(base)
@@ -1367,9 +1390,11 @@ class ipwd(Table):
     """
     List all entries in the Unix user account and password database.
 
-    Example:
+    Example::
 
         >>> ipwd | isort("uid")
+        <IPython.Extensions.ipipe.isort key='uid' reverse=False at 0x849efec>
+        # random
     """
     def __iter__(self):
         for entry in pwd.getpwall():
@@ -1551,9 +1576,10 @@ class ienv(Table):
     """
     List environment variables.
 
-    Example:
+    Example::
 
         >>> ienv
+        <class 'IPython.Extensions.ipipe.ienv'>
     """
 
     def __iter__(self):
@@ -1568,9 +1594,62 @@ class ienv(Table):
             yield (astyle.style_default, repr(self))
 
 
+class ihist(Table):
+    """
+    IPython input history
+
+    Example::
+
+        >>> ihist
+        <class 'IPython.Extensions.ipipe.ihist'>
+        >>> ihist(True) # raw mode
+        <IPython.Extensions.ipipe.ihist object at 0x849602c>  # random
+    """
+    def __init__(self, raw=True):
+        self.raw = raw
+
+    def __iter__(self):
+        api = ipapi.get()
+        if self.raw:
+            for line in api.IP.input_hist_raw:
+                yield line.rstrip("\n")
+        else:
+            for line in api.IP.input_hist:
+                yield line.rstrip("\n")
+
+
+class Alias(object):
+    """
+    Entry in the alias table
+    """
+    def __init__(self, name, args, command):
+        self.name = name
+        self.args = args
+        self.command = command
+
+    def __xattrs__(self, mode="default"):
+        return ("name", "args", "command")
+
+
+class ialias(Table):
+    """
+    IPython alias list
+
+    Example::
+
+        >>> ialias
+        <class 'IPython.Extensions.ipipe.ialias'>
+    """
+    def __iter__(self):
+        api = ipapi.get()
+
+        for (name, (args, command)) in api.IP.alias_table.iteritems():
+            yield Alias(name, args, command)
+
+
 class icsv(Pipe):
     """
-    This ``Pipe`` lists turn the input (with must be a pipe outputting lines
+    This ``Pipe`` turns the input (with must be a pipe outputting lines
     or an ``ifile``) into lines of CVS columns.
     """
     def __init__(self, **csvargs):
@@ -1619,10 +1698,14 @@ class ix(Table):
     Execute a system command and list its output as lines
     (similar to ``os.popen()``).
 
-    Examples:
+    Examples::
 
         >>> ix("ps x")
+        IPython.Extensions.ipipe.ix('ps x')
+
         >>> ix("find .") | ifile
+        <IPython.Extensions.ipipe.ieval expr=<class 'IPython.Extensions.ipipe.ifile'> at 0x8509d2c>
+        # random
     """
     def __init__(self, cmd):
         self.cmd = cmd
@@ -1658,11 +1741,12 @@ class ifilter(Pipe):
     Filter an input pipe. Only objects where an expression evaluates to true
     (and doesn't raise an exception) are listed.
 
-    Examples:
+    Examples::
 
         >>> ils | ifilter("_.isfile() and size>1000")
         >>> igrp | ifilter("len(mem)")
         >>> sys.modules | ifilter(lambda _:_.value is not None)
+        # all-random
     """
 
     def __init__(self, expr, globals=None, errors="raiseifallfail"):
@@ -1673,16 +1757,21 @@ class ifilter(Pipe):
         user namespace). ``errors`` specifies how exception during evaluation
         of ``expr`` are handled:
 
-        * ``drop``: drop all items that have errors;
+        ``"drop"``
+            drop all items that have errors;
 
-        * ``keep``: keep all items that have errors;
+        ``"keep"``
+            keep all items that have errors;
 
-        * ``keeperror``: keep the exception of all items that have errors;
+        ``"keeperror"``
+            keep the exception of all items that have errors;
 
-        * ``raise``: raise the exception;
+        ``"raise"``
+            raise the exception;
 
-        * ``raiseifallfail``: raise the first exception if all items have errors;
-          otherwise drop those with errors (this is the default).
+        ``"raiseifallfail"``
+            raise the first exception if all items have errors; otherwise drop
+            those with errors (this is the default).
         """
         self.expr = expr
         self.globals = globals
@@ -1745,10 +1834,12 @@ class ieval(Pipe):
     """
     Evaluate an expression for each object in the input pipe.
 
-    Examples:
+    Examples::
 
         >>> ils | ieval("_.abspath()")
+        # random
         >>> sys.path | ieval(ifile)
+        # random
     """
 
     def __init__(self, expr, globals=None, errors="raiseifallfail"):
@@ -1817,10 +1908,12 @@ class ienum(Pipe):
     Enumerate the input pipe (i.e. wrap each input object in an object
     with ``index`` and ``object`` attributes).
 
-    Examples:
+    Examples::
 
         >>> xrange(20) | ieval("_,_*_") | ienum | ifilter("index % 2 == 0") | ieval("object")
     """
+    skip_doctest = True
+    
     def __iter__(self):
         fields = ("index", "object")
         for (index, object) in enumerate(xiter(self.input)):
@@ -1831,10 +1924,13 @@ class isort(Pipe):
     """
     Sorts the input pipe.
 
-    Examples:
+    Examples::
 
         >>> ils | isort("size")
+        <IPython.Extensions.ipipe.isort key='size' reverse=False at 0x849ec2c>
         >>> ils | isort("_.isdir(), _.lower()", reverse=True)
+        <IPython.Extensions.ipipe.isort key='_.isdir(), _.lower()' reverse=True at 0x849eacc>
+        # all-random
     """
 
     def __init__(self, key=None, globals=None, reverse=False):
@@ -1910,6 +2006,9 @@ class Display(object):
         def __ror__(self, input):
             return input | self()
 
+    def __init__(self, input=None):
+        self.input = input
+
     def __ror__(self, input):
         self.input = input
         return self
@@ -1944,6 +2043,83 @@ class iless(Display):
             print "%s: %s" % (exc.__class__.__name__, str(exc))
 
 
+class _RedirectIO(object):
+    def __init__(self,*args,**kwargs):
+        """
+        Map the system output streams to self.
+        """
+        self.stream = StringIO.StringIO()
+        self.stdout = sys.stdout
+        sys.stdout = self
+        self.stderr = sys.stderr
+        sys.stderr = self
+
+    def write(self, text):
+        """
+        Write both to screen and to self.
+        """
+        self.stream.write(text)
+        self.stdout.write(text)
+        if "\n" in text:
+            self.stdout.flush()
+
+    def writelines(self, lines):
+        """
+        Write lines both to screen and to self.
+        """
+        self.stream.writelines(lines)
+        self.stdout.writelines(lines)
+        self.stdout.flush()
+
+    def restore(self):
+        """
+        Restore the default system streams.
+        """
+        self.stdout.flush()
+        self.stderr.flush()
+        sys.stdout = self.stdout
+        sys.stderr = self.stderr
+
+
+class icap(Table):
+    """
+    Execute a python string and capture any output to stderr/stdout.
+
+    Examples::
+
+        >>> import time
+        >>> icap("for i in range(10): print i, time.sleep(0.1)")
+
+    """
+    skip_doctest = True
+    
+    def __init__(self, expr, globals=None):
+        self.expr = expr
+        self.globals = globals
+        log = _RedirectIO()
+        try:
+            exec(expr, getglobals(globals))
+        finally:
+            log.restore()
+        self.stream = log.stream
+
+    def __iter__(self):
+        self.stream.seek(0)
+        for line in self.stream:
+            yield line.rstrip("\r\n")
+
+    def __xrepr__(self, mode="default"):
+        if mode == "header" or mode == "footer":
+            yield (astyle.style_default,
+                   "%s(%r)" % (self.__class__.__name__, self.expr))
+        else:
+            yield (astyle.style_default, repr(self))
+
+    def __repr__(self):
+        return "%s.%s(%r)" % \
+            (self.__class__.__module__, self.__class__.__name__, self.expr)
+
+
 def xformat(value, mode, maxlength):
     align = None
     full = True
@@ -1972,6 +2148,9 @@ def xformat(value, mode, maxlength):
     return (align, width, text)
 
 
+
+import astyle
+
 class idump(Display):
     # The approximate maximum length of a column entry
     maxattrlength = 200
@@ -1979,7 +2158,8 @@ class idump(Display):
     # Style for column names
     style_header = astyle.Style.fromstr("white:black:bold")
 
-    def __init__(self, *attrs):
+    def __init__(self, input=None, *attrs):
+        Display.__init__(self, input)
         self.attrs = [upgradexattr(attr) for attr in attrs]
         self.headerpadchar = " "
         self.headersepchar = "|"
@@ -2102,11 +2282,11 @@ class AttributeDetail(Table):
 try:
     from ibrowse import ibrowse
 except ImportError:
-    # No curses (probably Windows)
+    # No curses (probably Windows) => try igrid
     try:
         from igrid import igrid
     except ImportError:
-        # no wx eithevn do => use ``idump`` as the default display.
+        # no wx either => use ``idump`` as the default display.
         defaultdisplay = idump
     else:
         defaultdisplay = igrid
@@ -2116,27 +2296,21 @@ else:
     __all__.append("ibrowse")
 
 
-# If we're running under IPython, install an IPython displayhook that
-# returns the object from Display.display(), else install a displayhook
+# If we're running under IPython, register our objects with IPython's
+# generic function ``result_display``, else install a displayhook
 # directly as sys.displayhook
-api = None
-if ipapi is not None:
-    try:
-        api = ipapi.get()
-    except AttributeError:
-        pass
+if generics is not None:
+    def display_display(obj):
+        return obj.display()
+    generics.result_display.when_type(Display)(display_display)
 
-if api is not None:
-    def displayhook(self, obj):
-        if isinstance(obj, type) and issubclass(obj, Table):
-            obj = obj()
-        if isinstance(obj, Table):
-            obj = obj | defaultdisplay
-        if isinstance(obj, Display):
-            return obj.display()
-        else:
-            raise ipapi.TryNext
-    api.set_hook("result_display", displayhook)
+    def display_tableobject(obj):
+        return display_display(defaultdisplay(obj))
+    generics.result_display.when_type(Table)(display_tableobject)
+
+    def display_tableclass(obj):
+        return display_tableobject(obj())
+    generics.result_display.when_type(Table.__metaclass__)(display_tableclass)
 else:
     def installdisplayhook():
         _originalhook = sys.displayhook
@@ -2144,7 +2318,7 @@ else:
             if isinstance(obj, type) and issubclass(obj, Table):
                 obj = obj()
             if isinstance(obj, Table):
-                obj = obj | defaultdisplay
+                obj = defaultdisplay(obj)
             if isinstance(obj, Display):
                 return obj.display()
             else:
