@@ -31,6 +31,8 @@ o.prompt_in2 ='   .\D.: '
 o.prompt_out = 'Out[\#]: '
 o.prompts_pad_left=1
 
+o.autoindent = 1
+
 o.readline_remove_delims="-/~"
 o.readline_merge_completions=1
 
@@ -53,7 +55,8 @@ set show-all-if-ambiguous on
 "\C-k": kill-line
 "\C-u": unix-line-discard"""
 
-for cmd in rlopts.split('\n'):
-    readline.parse_and_bind(cmd)
+if readline.have_readline:
+    for cmd in rlopts.split('\n'):
+        readline.parse_and_bind(cmd)
     
     
