@@ -5,6 +5,10 @@ in IPython for demonstrations.  With very simple markup (a few tags in
 comments), you can control points where the script stops executing and returns
 control to IPython.
 
+
+Provided classes
+================
+
 The classes are (see their docstrings for further details):
 
  - Demo: pure python demos
@@ -19,6 +23,31 @@ The classes are (see their docstrings for further details):
  - IPythonLineDemo: IPython version of the LineDemo class (the demo is
  executed a line at a time, but processed via IPython).
 
+ - ClearMixin: mixin to make Demo classes with less visual clutter.  It
+   declares an empty marquee and a pre_cmd that clears the screen before each
+   block (see Subclassing below).
+
+ - ClearDemo, ClearIPDemo: mixin-enabled versions of the Demo and IPythonDemo
+   classes.
+
+
+Subclassing
+===========
+
+The classes here all include a few methods meant to make customization by
+subclassing more convenient.  Their docstrings below have some more details:
+
+  - marquee(): generates a marquee to provide visible on-screen markers at each
+    block start and end.
+
+  - pre_cmd(): run right before the execution of each block.
+
+  - post_cmd(): run right after the execution of each block.  If the block
+    raises an exception, this is NOT called.
+    
+
+Operation
+=========
 
 The file is run in its own empty namespace (though you can pass it a string of
 arguments as if in a command line environment, and it will see those as
@@ -34,11 +63,17 @@ behavior.
 
 The supported tags are:
 
-# <demo> --- stop ---
+# <demo> stop
 
   Defines block boundaries, the points where IPython stops execution of the
   file and returns to the interactive prompt.
 
+  You can optionally mark the stop tag with extra dashes before and after the
+  word 'stop', to help visually distinguish the blocks in a text editor:
+
+  # <demo> --- stop ---
+  
+
 # <demo> silent
 
   Make a block execute silently (and hence automatically).  Typically used in
@@ -76,27 +111,41 @@ has a few useful methods for navigation, like again(), edit(), jump(), seek()
 and back().  It can be reset for a new run via reset() or reloaded from disk
 (in case you've edited the source) via reload().  See their docstrings below.
 
+Note: To make this simpler to explore, a file called "demo-exercizer.py" has
+been added to the "docs/examples/core" directory.  Just cd to this directory in
+an IPython session, and type::
+
+  %run demo-exercizer.py
+  
+and then follow the directions.
+
+Example
+=======
+
+The following is a very simple example of a valid demo file.
+
 #################### EXAMPLE DEMO <ex_demo.py> ###############################
 '''A simple interactive demo to illustrate the use of IPython's Demo class.'''
 
 print 'Hello, welcome to an interactive IPython demo.'
 
 # The mark below defines a block boundary, which is a point where IPython will
-# stop execution and return to the interactive prompt.
-# Note that in actual interactive execution, 
-# <demo> --- stop ---
+# stop execution and return to the interactive prompt. The dashes are actually
+# optional and used only as a visual aid to clearly separate blocks while
+# editing the demo code.
+# <demo> stop
 
 x = 1
 y = 2
 
-# <demo> --- stop ---
+# <demo> stop
 
 # the mark below makes this block as silent
 # <demo> silent
 
 print 'This is a silent block, which gets executed but not printed.'
 
-# <demo> --- stop ---
+# <demo> stop
 # <demo> auto
 print 'This is an automatic block.'
 print 'It is executed without asking for confirmation, but printed.'
@@ -104,13 +153,14 @@ z = x+y
 
 print 'z=',x
 
-# <demo> --- stop ---
+# <demo> stop
 # This is just another normal block.
 print 'z is now:', z
 
 print 'bye!'
 ################### END EXAMPLE DEMO <ex_demo.py> ############################
 """
+
 #*****************************************************************************
 #     Copyright (C) 2005-2006 Fernando Perez. <Fernando.Perez@colorado.edu>
 #
@@ -126,7 +176,7 @@ import shlex
 import sys
 
 from IPython.PyColorize import Parser
-from IPython.genutils import marquee, file_read, file_readlines
+from IPython.genutils import marquee, file_read, file_readlines, Term
 
 __all__ = ['Demo','IPythonDemo','LineDemo','IPythonLineDemo','DemoError']
 
@@ -135,14 +185,14 @@ class DemoError(exceptions.Exception): pass
 def re_mark(mark):
     return re.compile(r'^\s*#\s+<demo>\s+%s\s*$' % mark,re.MULTILINE)
 
-class Demo:
+class Demo(object):
 
-    re_stop     = re_mark('---\s?stop\s?---')
+    re_stop     = re_mark('-*\s?stop\s?-*')
     re_silent   = re_mark('silent')
     re_auto     = re_mark('auto')
     re_auto_all = re_mark('auto_all')
 
-    def __init__(self,fname,arg_str='',auto_all=None):
+    def __init__(self,src,title='',arg_str='',auto_all=None):
         """Make a new demo object.  To run the demo, simply call the object.
 
         See the module docstring for full details and an example (you can use
@@ -150,9 +200,14 @@ class Demo:
 
         Inputs:
         
-          - fname = filename.
+          - src is either a file, or file-like object, or a
+              string that can be resolved to a filename.
 
         Optional inputs:
+        
+          - title: a string to use as the demo name.  Of most use when the demo
+          you are making comes from an object that has no filename, or if you 
+          want an alternate denotation distinct from the filename.
 
           - arg_str(''): a string of arguments, internally converted to a list
           just like sys.argv, so the demo script can see a similar
@@ -164,9 +219,24 @@ class Demo:
           can be changed at runtime simply by reassigning it to a boolean
           value.
           """
-        
-        self.fname    = fname
-        self.sys_argv = [fname] + shlex.split(arg_str)
+        if hasattr(src, "read"):
+             # It seems to be a file or a file-like object
+            self.fobj = src
+            self.fname = "from a file-like object"
+            if title == '':
+                self.title = "from a file-like object"
+            else:
+                self.title = title
+        else:
+             # Assume it's a string or something that can be converted to one
+            self.fobj = open(src)
+            self.fname = src
+            if title == '':
+                (filepath, filename) = os.path.split(src)
+                self.title = filename
+            else:
+                self.title = title
+        self.sys_argv = [src] + shlex.split(arg_str)
         self.auto_all = auto_all
         
         # get a few things from ipython.  While it's a bit ugly design-wise,
@@ -185,7 +255,7 @@ class Demo:
     def reload(self):
         """Reload source from disk and initialize state."""
         # read data and parse into blocks
-        self.src     = file_read(self.fname)
+        self.src     = self.fobj.read()
         src_b        = [b.strip() for b in self.re_stop.split(self.src) if b]
         self._silent = [bool(self.re_silent.findall(b)) for b in src_b]
         self._auto   = [bool(self.re_auto.findall(b)) for b in src_b]
@@ -234,7 +304,7 @@ class Demo:
         
         if index is None:
             if self.finished:
-                print 'Demo finished.  Use reset() if you want to rerun it.'
+                print >>Term.cout, 'Demo finished.  Use <demo_name>.reset() if you want to rerun it.'
                 return None
             index = self.block_index
         else:
@@ -242,7 +312,12 @@ class Demo:
         return index
 
     def seek(self,index):
-        """Move the current seek pointer to the given block"""
+        """Move the current seek pointer to the given block.
+
+        You can use negative indices to seek from the end, with identical
+        semantics to those of Python lists."""
+        if index<0:
+            index = self.nblocks + index
         self._validate_index(index)
         self.block_index = index
         self.finished = False
@@ -251,8 +326,10 @@ class Demo:
         """Move the seek pointer back num blocks (default is 1)."""
         self.seek(self.block_index-num)
 
-    def jump(self,num):
-        """Jump a given number of blocks relative to the current one."""
+    def jump(self,num=1):
+        """Jump a given number of blocks relative to the current one.
+
+        The offset can be positive or negative, defaults to 1."""
         self.seek(self.block_index+num)
 
     def again(self):
@@ -296,25 +373,27 @@ class Demo:
         if index is None:
             return
 
-        print marquee('<%s> block # %s (%s remaining)' %
-                      (self.fname,index,self.nblocks-index-1))
-        print self.src_blocks_colored[index],
+        print >>Term.cout, self.marquee('<%s> block # %s (%s remaining)' %
+                           (self.title,index,self.nblocks-index-1))
+        print >>Term.cout,(self.src_blocks_colored[index])
         sys.stdout.flush()
 
     def show_all(self):
         """Show entire demo on screen, block by block"""
 
-        fname = self.fname
+        fname = self.title
+        title = self.title
         nblocks = self.nblocks
         silent = self._silent
+        marquee = self.marquee
         for index,block in enumerate(self.src_blocks_colored):
             if silent[index]:
-                print marquee('<%s> SILENT block # %s (%s remaining)' %
-                              (fname,index,nblocks-index-1))
+                print >>Term.cout, marquee('<%s> SILENT block # %s (%s remaining)' %
+                              (title,index,nblocks-index-1))
             else:
-                print marquee('<%s> block # %s (%s remaining)' %
-                              (fname,index,nblocks-index-1))
-            print block,
+                print >>Term.cout, marquee('<%s> block # %s (%s remaining)' %
+                              (title,index,nblocks-index-1))
+            print >>Term.cout, block,
         sys.stdout.flush()
 
     def runlines(self,source):
@@ -335,25 +414,28 @@ class Demo:
         if index is None:
             return
         try:
+            marquee = self.marquee
             next_block = self.src_blocks[index]
             self.block_index += 1
             if self._silent[index]:
-                print marquee('Executing silent block # %s (%s remaining)' %
+                print >>Term.cout, marquee('Executing silent block # %s (%s remaining)' %
                               (index,self.nblocks-index-1))
             else:
+                self.pre_cmd()
                 self.show(index)
                 if self.auto_all or self._auto[index]:
-                    print marquee('output')
+                    print >>Term.cout, marquee('output:')
                 else:
-                    print marquee('Press <q> to quit, <Enter> to execute...'),
+                    print >>Term.cout, marquee('Press <q> to quit, <Enter> to execute...'),
                     ans = raw_input().strip()
                     if ans:
-                        print marquee('Block NOT executed')
+                        print >>Term.cout, marquee('Block NOT executed')
                         return
             try:
                 save_argv = sys.argv
                 sys.argv = self.sys_argv
                 self.runlines(next_block)
+                self.post_cmd()
             finally:
                 sys.argv = save_argv
             
@@ -363,11 +445,29 @@ class Demo:
             self.ip_ns.update(self.user_ns)
 
         if self.block_index == self.nblocks:
-            print
-            print marquee(' END OF DEMO ')
-            print marquee('Use reset() if you want to rerun it.')
+            mq1 = self.marquee('END OF DEMO')
+            if mq1:
+                # avoid spurious print >>Term.cout,s if empty marquees are used
+                print >>Term.cout
+                print >>Term.cout, mq1
+                print >>Term.cout, self.marquee('Use <demo_name>.reset() if you want to rerun it.')
             self.finished = True
 
+    # These methods are meant to be overridden by subclasses who may wish to
+    # customize the behavior of of their demos.
+    def marquee(self,txt='',width=78,mark='*'):
+        """Return the input string centered in a 'marquee'."""
+        return marquee(txt,width,mark)
+
+    def pre_cmd(self):
+        """Method called before executing each block."""
+        pass
+
+    def post_cmd(self):
+        """Method called after executing each block."""
+        pass
+
+
 class IPythonDemo(Demo):
     """Class for interactive demos with IPython's input processing applied.
 
@@ -384,7 +484,7 @@ class IPythonDemo(Demo):
     def runlines(self,source):
         """Execute a string with one or more lines of code"""
 
-        self.runlines(source)
+        self.shell.runlines(source)
         
 class LineDemo(Demo):
     """Demo where each line is executed as a separate block.
@@ -399,9 +499,9 @@ class LineDemo(Demo):
     def reload(self):
         """Reload source from disk and initialize state."""
         # read data and parse into blocks
-        src_b           = [l for l in file_readlines(self.fname) if l.strip()]
+        src_b           = [l for l in self.fobj.readline() if l.strip()]
         nblocks         = len(src_b)
-        self.src        = os.linesep.join(file_readlines(self.fname))
+        self.src        = os.linesep.join(self.fobj.readlines())
         self._silent    = [False]*nblocks
         self._auto      = [True]*nblocks
         self.auto_all   = True
@@ -414,6 +514,41 @@ class LineDemo(Demo):
         # ensure clean namespace and seek offset
         self.reset()
 
+
 class IPythonLineDemo(IPythonDemo,LineDemo):
     """Variant of the LineDemo class whose input is processed by IPython."""
     pass
+
+
+class ClearMixin(object):
+    """Use this mixin to make Demo classes with less visual clutter.
+    
+    Demos using this mixin will clear the screen before every block and use
+    blank marquees.
+    
+    Note that in order for the methods defined here to actually override those
+    of the classes it's mixed with, it must go /first/ in the inheritance
+    tree.  For example:
+    
+        class ClearIPDemo(ClearMixin,IPythonDemo): pass
+    
+    will provide an IPythonDemo class with the mixin's features.
+    """
+    
+    def marquee(self,txt='',width=78,mark='*'):
+        """Blank marquee that returns '' no matter what the input."""
+        return ''
+    
+    def pre_cmd(self):
+        """Method called before executing each block.
+        
+        This one simply clears the screen."""
+        import IPython.platutils
+        IPython.platutils.term_clear()
+
+class ClearDemo(ClearMixin,Demo):
+    pass
+
+
+class ClearIPDemo(ClearMixin,IPythonDemo):
+    pass
