@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# -*- coding: iso-8859-15 -*-
+# -*- coding: utf-8 -*-
 '''
 Provides IPython WX console widgets.
 
@@ -19,7 +19,7 @@ available under the terms of the BSD which accompanies this distribution, and
 is available at U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
-__version__ = 0.8
+__version__ = 0.9
 __author__  = "Laurent Dufrechou"
 __email__   = "laurent.dufrechou _at_ gmail.com"
 __license__ = "BSD"
@@ -28,13 +28,24 @@ import wx
 import wx.stc  as  stc
 
 import re
-import sys
-import locale
 from StringIO import StringIO
-try:
-        import IPython
-except Exception,e:
-        raise "Error importing IPython (%s)" % str(e)
+
+import sys
+import codecs
+import locale
+import time
+
+for enc in (locale.getpreferredencoding(),
+            sys.getfilesystemencoding(),
+            sys.getdefaultencoding()):
+    try:
+        codecs.lookup(enc)
+        ENCODING = enc
+        break
+    except LookupError:
+        pass
+else:
+    ENCODING = 'utf-8'
 
 from ipshell_nonblocking import NonBlockingIPShell
 
@@ -47,24 +58,50 @@ class WxNonBlockingIPShell(NonBlockingIPShell):
                  cin=None, cout=None, cerr=None,
                  ask_exit_handler=None):
         
-        NonBlockingIPShell.__init__(self,argv,user_ns,user_global_ns,
+        NonBlockingIPShell.__init__(self, argv, user_ns, user_global_ns,
                                     cin, cout, cerr,
                                     ask_exit_handler)
 
         self.parent = parent
 
         self.ask_exit_callback = ask_exit_handler
-        self._IP.exit = self._askExit
+        self._IP.exit = self._ask_exit
 
-    def addGUIShortcut(self,text,func):
+    def addGUIShortcut(self, text, func):
         wx.CallAfter(self.parent.add_button_handler, 
                 button_info={   'text':text, 
                                 'func':self.parent.doExecuteLine(func)})
 
-    def _askExit(self):
+    def _raw_input(self, prompt=''):
+        """ A replacement from python's raw_input.
+        """
+        self.answer = None
+        if(self._threading == True):                    
+                        wx.CallAfter(self._yesNoBox,  prompt)
+                        while self.answer is None:
+                                time.sleep(.1)
+        else:
+                        self._yesNoBox(prompt)
+        return self.answer
+        
+    def _yesNoBox(self, prompt):
+        """ yes/no box managed with wx.CallAfter jsut in case caler is executed in a thread"""
+        dlg = wx.TextEntryDialog(
+                self.parent, prompt,
+                'Input requested', 'Python')
+        dlg.SetValue("")
+
+        answer = ''
+        if dlg.ShowModal() == wx.ID_OK:
+            answer = dlg.GetValue()
+        
+        dlg.Destroy()
+        self.answer = answer
+        
+    def _ask_exit(self):
         wx.CallAfter(self.ask_exit_callback, ())
 
-    def _afterExecute(self):
+    def _after_execute(self):
         wx.CallAfter(self.parent.evtStateExecuteDone, ())
 
                 
@@ -85,27 +122,27 @@ class WxConsoleView(stc.StyledTextCtrl):
     @ivar color_pat: Regex of terminal color pattern
     @type color_pat: _sre.SRE_Pattern
     '''
-    ANSI_STYLES_BLACK={'0;30': [0,'WHITE'],            '0;31': [1,'RED'],
-                       '0;32': [2,'GREEN'],            '0;33': [3,'BROWN'],
-                       '0;34': [4,'BLUE'],             '0;35': [5,'PURPLE'],
-                       '0;36': [6,'CYAN'],             '0;37': [7,'LIGHT GREY'],
-                       '1;30': [8,'DARK GREY'],        '1;31': [9,'RED'],
-                       '1;32': [10,'SEA GREEN'],       '1;33': [11,'YELLOW'],
-                       '1;34': [12,'LIGHT BLUE'],      '1;35': 
-                                                    [13,'MEDIUM VIOLET RED'],
-                       '1;36': [14,'LIGHT STEEL BLUE'],'1;37': [15,'YELLOW']}
+    ANSI_STYLES_BLACK = {'0;30': [0, 'WHITE'],            '0;31': [1, 'RED'],
+                         '0;32': [2, 'GREEN'],            '0;33': [3, 'BROWN'],
+                         '0;34': [4, 'BLUE'],             '0;35': [5, 'PURPLE'],
+                         '0;36': [6, 'CYAN'],             '0;37': [7, 'LIGHT GREY'],
+                         '1;30': [8, 'DARK GREY'],        '1;31': [9, 'RED'],
+                         '1;32': [10, 'SEA GREEN'],       '1;33': [11, 'YELLOW'],
+                         '1;34': [12, 'LIGHT BLUE'],      '1;35': 
+                                                          [13, 'MEDIUM VIOLET RED'],
+                         '1;36': [14, 'LIGHT STEEL BLUE'], '1;37': [15, 'YELLOW']}
 
-    ANSI_STYLES_WHITE={'0;30': [0,'BLACK'],            '0;31': [1,'RED'],
-                       '0;32': [2,'GREEN'],            '0;33': [3,'BROWN'],
-                       '0;34': [4,'BLUE'],             '0;35': [5,'PURPLE'],
-                       '0;36': [6,'CYAN'],             '0;37': [7,'LIGHT GREY'],
-                       '1;30': [8,'DARK GREY'],        '1;31': [9,'RED'],
-                       '1;32': [10,'SEA GREEN'],       '1;33': [11,'YELLOW'],
-                       '1;34': [12,'LIGHT BLUE'],      '1;35':
-                                                    [13,'MEDIUM VIOLET RED'],
-                       '1;36': [14,'LIGHT STEEL BLUE'],'1;37': [15,'YELLOW']}
+    ANSI_STYLES_WHITE = {'0;30': [0, 'BLACK'],            '0;31': [1, 'RED'],
+                         '0;32': [2, 'GREEN'],            '0;33': [3, 'BROWN'],
+                         '0;34': [4, 'BLUE'],             '0;35': [5, 'PURPLE'],
+                         '0;36': [6, 'CYAN'],             '0;37': [7, 'LIGHT GREY'],
+                         '1;30': [8, 'DARK GREY'],        '1;31': [9, 'RED'],
+                         '1;32': [10, 'SEA GREEN'],       '1;33': [11, 'YELLOW'],
+                         '1;34': [12, 'LIGHT BLUE'],      '1;35':
+                                                           [13, 'MEDIUM VIOLET RED'],
+                         '1;36': [14, 'LIGHT STEEL BLUE'], '1;37': [15, 'YELLOW']}
 
-    def __init__(self,parent,prompt,intro="",background_color="BLACK",
+    def __init__(self, parent, prompt, intro="", background_color="BLACK",
                  pos=wx.DefaultPosition, ID = -1, size=wx.DefaultSize,
                  style=0, autocomplete_mode = 'IPYTHON'):
         '''
@@ -146,14 +183,17 @@ class WxConsoleView(stc.StyledTextCtrl):
         #self.SetUseAntiAliasing(True)
         self.SetLayoutCache(stc.STC_CACHE_PAGE)
         self.SetUndoCollection(False)
+        self.SetUseTabs(True)
+        self.SetIndent(4)
+        self.SetTabWidth(4)
 
         self.EnsureCaretVisible()
         
-        self.SetMargins(3,3) #text is moved away from border with 3px
+        self.SetMargins(3, 3) #text is moved away from border with 3px
         # Suppressing Scintilla margins
-        self.SetMarginWidth(0,0)
-        self.SetMarginWidth(1,0)
-        self.SetMarginWidth(2,0)
+        self.SetMarginWidth(0, 0)
+        self.SetMarginWidth(1, 0)
+        self.SetMarginWidth(2, 0)
 
         self.background_color = background_color
         self.buildStyles()
@@ -173,13 +213,13 @@ class WxConsoleView(stc.StyledTextCtrl):
     def buildStyles(self):
         #we define platform specific fonts
         if wx.Platform == '__WXMSW__':
-                faces = { 'times': 'Times New Roman',
-                          'mono' : 'Courier New',
-                          'helv' : 'Arial',
-                          'other': 'Comic Sans MS',
-                          'size' : 10,
-                          'size2': 8,
-                         }
+            faces = { 'times': 'Times New Roman',
+                      'mono' : 'Courier New',
+                      'helv' : 'Arial',
+                      'other': 'Comic Sans MS',
+                      'size' : 10,
+                      'size2': 8,
+                     }
         elif wx.Platform == '__WXMAC__':
             faces = { 'times': 'Times New Roman',
                       'mono' : 'Monaco',
@@ -222,11 +262,11 @@ class WxConsoleView(stc.StyledTextCtrl):
         
         #######################################################################
         
-    def setBackgroundColor(self,color):
+    def setBackgroundColor(self, color):
         self.background_color = color
         self.buildStyles()
 
-    def getBackgroundColor(self,color):
+    def getBackgroundColor(self, color):
         return self.background_color
         
     def asyncWrite(self, text):
@@ -237,22 +277,15 @@ class WxConsoleView(stc.StyledTextCtrl):
         @type text: string
         '''
         try:
-                #print >>sys.__stdout__,'entering'
-                wx.MutexGuiEnter()
-                #print >>sys.__stdout__,'locking the GUI'
+            wx.MutexGuiEnter()
                 
-                #be sure not to be interrutpted before the MutexGuiLeave!
-                self.write(text)
-                
-                #print >>sys.__stdout__,'done'
+            #be sure not to be interrutpted before the MutexGuiLeave!
+            self.write(text)
                 
         except KeyboardInterrupt:
-                #print >>sys.__stdout__,'got keyboard interrupt'
-                wx.MutexGuiLeave()
-                #print >>sys.__stdout__,'interrupt unlock the GUI'
-                raise KeyboardInterrupt
+            wx.MutexGuiLeave()
+            raise KeyboardInterrupt
         wx.MutexGuiLeave()
-        #print >>sys.__stdout__,'normal unlock the GUI'
         
                 
     def write(self, text):
@@ -264,7 +297,7 @@ class WxConsoleView(stc.StyledTextCtrl):
         '''
         segments = self.color_pat.split(text)
         segment = segments.pop(0)
-        self.StartStyling(self.getCurrentLineEnd(),0xFF)
+        self.StartStyling(self.getCurrentLineEnd(), 0xFF)
         self.AppendText(segment)
         
         if segments:
@@ -272,11 +305,11 @@ class WxConsoleView(stc.StyledTextCtrl):
 
             for tag in ansi_tags:
                 i = segments.index(tag)
-                self.StartStyling(self.getCurrentLineEnd(),0xFF)
+                self.StartStyling(self.getCurrentLineEnd(), 0xFF)
                 self.AppendText(segments[i+1])
 
                 if tag != '0':
-                    self.SetStyling(len(segments[i+1]),self.ANSI_STYLES[tag][0])
+                    self.SetStyling(len(segments[i+1]), self.ANSI_STYLES[tag][0])
 
                 segments.pop(i)
                 
@@ -288,13 +321,13 @@ class WxConsoleView(stc.StyledTextCtrl):
         '''
         return len(str(self.prompt_count)) + 7
 
-    def setPrompt(self,prompt):
+    def setPrompt(self, prompt):
         self.prompt = prompt
 
-    def setIndentation(self,indentation):
+    def setIndentation(self, indentation):
         self.indent = indentation
         
-    def setPromptCount(self,count):
+    def setPromptCount(self, count):
         self.prompt_count = count
         
     def showPrompt(self):
@@ -319,7 +352,7 @@ class WxConsoleView(stc.StyledTextCtrl):
         @param text: Text to use as replacement.
         @type text: string
         '''
-        self.SetSelection(self.getCurrentPromptStart(),self.getCurrentLineEnd())
+        self.SetSelection(self.getCurrentPromptStart(), self.getCurrentLineEnd())
         self.ReplaceSelection(text)
         self.moveCursor(self.getCurrentLineEnd())
 
@@ -347,30 +380,30 @@ class WxConsoleView(stc.StyledTextCtrl):
         if self.GetCurrentPos() < self.getCurrentPromptStart():
             self.GotoPos(self.getCurrentPromptStart())
         
-    def removeFromTo(self,from_pos,to_pos):
+    def removeFromTo(self, from_pos, to_pos):
         if from_pos < to_pos:
-            self.SetSelection(from_pos,to_pos)
+            self.SetSelection(from_pos, to_pos)
             self.DeleteBack()
                                                           
     def removeCurrentLine(self):
         self.LineDelete()
         
-    def moveCursor(self,position):
+    def moveCursor(self, position):
         self.GotoPos(position)
 
     def getCursorPos(self):
         return self.GetCurrentPos()
 
-    def selectFromTo(self,from_pos,to_pos):
+    def selectFromTo(self, from_pos, to_pos):
         self.SetSelectionStart(from_pos)
         self.SetSelectionEnd(to_pos)
         
-    def writeHistory(self,history):
-        self.removeFromTo(self.getCurrentPromptStart(),self.getCurrentLineEnd())
+    def writeHistory(self, history):
+        self.removeFromTo(self.getCurrentPromptStart(), self.getCurrentLineEnd())
         self.changeLine(history)
 
     def setCompletionMethod(self, completion):
-        if completion in ['IPYTHON','STC']:
+        if completion in ['IPYTHON', 'STC']:
             self.autocomplete_mode = completion
         else:
             raise AttributeError
@@ -380,28 +413,26 @@ class WxConsoleView(stc.StyledTextCtrl):
         
     def writeCompletion(self, possibilities):
         if self.autocomplete_mode == 'IPYTHON':
-            max_len = len(max(possibilities,key=len))
-            max_symbol =' '*max_len
+            max_len = len(max(possibilities, key=len))
+            max_symbol = ' '*max_len
             
             #now we check how much symbol we can put on a line...
-            cursor_pos = self.getCursorPos()
             test_buffer = max_symbol + ' '*4
-            current_lines = self.GetLineCount()
             
             allowed_symbols = 80/len(test_buffer)
             if allowed_symbols == 0:
-                    allowed_symbols = 1
+                allowed_symbols = 1
             
             pos = 1
             buf = ''
             for symbol in possibilities:
                 #buf += symbol+'\n'#*spaces)
-                if pos<allowed_symbols:
+                if pos < allowed_symbols:
                     spaces = max_len - len(symbol) + 4
                     buf += symbol+' '*spaces
                     pos += 1
                 else:
-                    buf+=symbol+'\n'
+                    buf += symbol+'\n'
                     pos = 1
             self.write(buf)
         else:
@@ -409,7 +440,7 @@ class WxConsoleView(stc.StyledTextCtrl):
             self.AutoCompSetIgnoreCase(False)
             self.AutoCompSetAutoHide(False)
             #let compute the length ot last word
-            splitter = [' ','(','[','{']
+            splitter = [' ', '(', '[', '{','=']
             last_word = self.getCurrentLine()
             for breaker in splitter:
                 last_word = last_word.split(breaker)[-1]
@@ -429,7 +460,6 @@ class WxConsoleView(stc.StyledTextCtrl):
         @return: Return True if event as been catched.
         @rtype: boolean
         '''
-
         if not self.AutoCompActive():
             if event.GetKeyCode() == wx.WXK_HOME:
                 if event.Modifiers == wx.MOD_NONE:
@@ -438,7 +468,7 @@ class WxConsoleView(stc.StyledTextCtrl):
                     return True
                 elif event.Modifiers == wx.MOD_SHIFT:
                     self.moveCursorOnNewValidKey()
-                    self.selectFromTo(self.getCurrentPromptStart(),self.getCursorPos())
+                    self.selectFromTo(self.getCurrentPromptStart(), self.getCursorPos())
                     return True
                 else:
                     return False
@@ -459,7 +489,8 @@ class WxConsoleView(stc.StyledTextCtrl):
                 return True
             
             if skip:
-                if event.GetKeyCode() not in [wx.WXK_PAGEUP,wx.WXK_PAGEDOWN] and event.Modifiers == wx.MOD_NONE:
+                if event.GetKeyCode() not in [wx.WXK_PAGEUP, wx.WXK_PAGEDOWN]\
+                and event.Modifiers == wx.MOD_NONE:
                     self.moveCursorOnNewValidKey()
                     
                 event.Skip()
@@ -543,24 +574,30 @@ class IPShellWidget(wx.Panel):
         #with intro=''
         if intro is None:
             welcome_text = "Welcome to WxIPython Shell.\n\n"
-            welcome_text+= self.IP.getBanner()
+            welcome_text+= self.IP.get_banner()
             welcome_text+= "!command  -> Execute command in shell\n"
             welcome_text+= "TAB       -> Autocompletion\n"
         else:
             welcome_text = intro
 
         self.text_ctrl = WxConsoleView(self,
-                                       self.IP.getPrompt(),
+                                       self.IP.get_prompt(),
                                        intro=welcome_text,
                                        background_color=background_color)
 
-        self.cout.write = self.text_ctrl.asyncWrite
-
         option_text = wx.StaticText(self, -1, "Options:")
         self.completion_option = wx.CheckBox(self, -1, "Scintilla Completion")
+        self.completion_option.SetToolTip(wx.ToolTip(
+            "Selects the completion type:\nEither Ipython default style or Scintilla one"))
         #self.completion_option.SetValue(False)
         self.background_option = wx.CheckBox(self, -1, "White Background")
+        self.background_option.SetToolTip(wx.ToolTip(
+            "Selects the back ground color: BLACK or WHITE"))
         #self.background_option.SetValue(False)
+        self.threading_option = wx.CheckBox(self, -1, "Execute in thread")
+        self.threading_option.SetToolTip(wx.ToolTip(
+            "Use threading: infinite loop don't freeze the GUI and commands can be breaked\nNo  threading: maximum compatibility"))
+        #self.threading_option.SetValue(False)
         
         self.options={'completion':{'value':'IPYTHON',
                                     'checkbox':self.completion_option,'STC':True,'IPYTHON':False,
@@ -568,12 +605,20 @@ class IPShellWidget(wx.Panel):
                       'background_color':{'value':'BLACK',
                                           'checkbox':self.background_option,'WHITE':True,'BLACK':False,
                                           'setfunc':self.text_ctrl.setBackgroundColor},
+                      'threading':{'value':'True',
+                                   'checkbox':self.threading_option,'True':True,'False':False,
+                                   'setfunc':self.IP.set_threading},
                      }
+
+        #self.cout.write dEfault option is asynchroneous because default sate is threading ON
+        self.cout.write = self.text_ctrl.asyncWrite
+        #we reloard options
         self.reloadOptions(self.options)
         
         self.text_ctrl.Bind(wx.EVT_KEY_DOWN, self.keyPress)
         self.completion_option.Bind(wx.EVT_CHECKBOX, self.evtCheckOptionCompletion)
         self.background_option.Bind(wx.EVT_CHECKBOX, self.evtCheckOptionBackgroundColor)
+        self.threading_option.Bind(wx.EVT_CHECKBOX, self.evtCheckOptionThreading)
             
         ### making the layout of the panel ###
         sizer = wx.BoxSizer(wx.VERTICAL)
@@ -585,7 +630,9 @@ class IPShellWidget(wx.Panel):
                               (5, 5),
                               (self.completion_option, 0, wx.ALIGN_CENTER_VERTICAL),
                               (8, 8),
-                              (self.background_option, 0, wx.ALIGN_CENTER_VERTICAL)
+                              (self.background_option, 0, wx.ALIGN_CENTER_VERTICAL),
+                              (8, 8),
+                              (self.threading_option, 0, wx.ALIGN_CENTER_VERTICAL)
                               ])
         self.SetAutoLayout(True)
         sizer.Fit(self)
@@ -608,13 +655,15 @@ class IPShellWidget(wx.Panel):
         self.text_ctrl.write('\n')
         lines_to_execute = lines.replace('\t',' '*4)
         lines_to_execute = lines_to_execute.replace('\r','')
-        self.IP.doExecute(lines_to_execute.encode('cp1252'))
+        self.IP.do_execute(lines_to_execute.encode(ENCODING))
         self.updateHistoryTracker(lines)
+        if(self.text_ctrl.getCursorPos()!=0):
+            self.text_ctrl.removeCurrentLine()
         self.setCurrentState('WAIT_END_OF_EXECUTION')
         
     def evtStateExecuteDone(self,evt):
-        self.doc = self.IP.getDocText()
-        self.help = self.IP.getHelpText()
+        self.doc = self.IP.get_doc_text()
+        self.help = self.IP.get_help_text()
         if self.doc:
             self.pager_lines = self.doc[7:].split('\n')
             self.pager_state = 'INIT'
@@ -626,15 +675,17 @@ class IPShellWidget(wx.Panel):
             self.setCurrentState('SHOW_DOC')
             self.pager(self.help)                
         else:
+            if(self.text_ctrl.getCursorPos()!=0):
+                self.text_ctrl.removeCurrentLine()
             self.stateShowPrompt()
 
     def stateShowPrompt(self):
         self.setCurrentState('SHOW_PROMPT')
-        self.text_ctrl.setPrompt(self.IP.getPrompt())
-        self.text_ctrl.setIndentation(self.IP.getIndentation())
-        self.text_ctrl.setPromptCount(self.IP.getPromptCount())
+        self.text_ctrl.setPrompt(self.IP.get_prompt())
+        self.text_ctrl.setIndentation(self.IP.get_indentation())
+        self.text_ctrl.setPromptCount(self.IP.get_prompt_count())
         self.text_ctrl.showPrompt()
-        self.IP.initHistoryIndex()
+        self.IP.init_history_index()
         self.setCurrentState('IDLE')
 
     def setCurrentState(self, state):
@@ -644,51 +695,51 @@ class IPShellWidget(wx.Panel):
     def pager(self,text):
 
         if self.pager_state == 'INIT':
-                #print >>sys.__stdout__,"PAGER state:",self.pager_state
-                self.pager_nb_lines = len(self.pager_lines)
-                self.pager_index = 0
-                self.pager_do_remove = False
-                self.text_ctrl.write('\n')
-                self.pager_state = 'PROCESS_LINES'
+            #print >>sys.__stdout__,"PAGER state:",self.pager_state
+            self.pager_nb_lines = len(self.pager_lines)
+            self.pager_index = 0
+            self.pager_do_remove = False
+            self.text_ctrl.write('\n')
+            self.pager_state = 'PROCESS_LINES'
 
         if self.pager_state == 'PROCESS_LINES':
-                #print >>sys.__stdout__,"PAGER state:",self.pager_state
-                if self.pager_do_remove == True:
-                        self.text_ctrl.removeCurrentLine()
-                        self.pager_do_remove = False
+            #print >>sys.__stdout__,"PAGER state:",self.pager_state
+            if self.pager_do_remove == True:
+                self.text_ctrl.removeCurrentLine()
+                self.pager_do_remove = False
 
-                if self.pager_nb_lines > 10:
-                        #print >>sys.__stdout__,"PAGER processing 10 lines"
-                        if self.pager_index > 0:
-                                self.text_ctrl.write(">\x01\x1b[1;36m\x02"+self.pager_lines[self.pager_index]+'\n')
-                        else:
-                                self.text_ctrl.write("\x01\x1b[1;36m\x02 "+self.pager_lines[self.pager_index]+'\n')
-                        
-                        for line in self.pager_lines[self.pager_index+1:self.pager_index+9]:
-                                self.text_ctrl.write("\x01\x1b[1;36m\x02 "+line+'\n')
-                        self.pager_index += 10
-                        self.pager_nb_lines -= 10
-                        self.text_ctrl.write("--- Push Enter to continue or 'Q' to quit---")
-                        self.pager_do_remove = True
-                        self.pager_state = 'WAITING'
-                        return
+            if self.pager_nb_lines > 10:
+                #print >>sys.__stdout__,"PAGER processing 10 lines"
+                if self.pager_index > 0:
+                    self.text_ctrl.write(">\x01\x1b[1;36m\x02"+self.pager_lines[self.pager_index]+'\n')
                 else:
-                        #print >>sys.__stdout__,"PAGER processing last lines"
-                        if self.pager_nb_lines > 0:
-                                if self.pager_index > 0:
-                                        self.text_ctrl.write(">\x01\x1b[1;36m\x02"+self.pager_lines[self.pager_index]+'\n')
-                                else:
-                                        self.text_ctrl.write("\x01\x1b[1;36m\x02 "+self.pager_lines[self.pager_index]+'\n')
-                                
-                                self.pager_index += 1
-                                self.pager_nb_lines -= 1
-                        if self.pager_nb_lines > 0:
-                                for line in self.pager_lines[self.pager_index:]:
-                                        self.text_ctrl.write("\x01\x1b[1;36m\x02 "+line+'\n')
-                                        self.pager_nb_lines = 0
-                        self.pager_state = 'DONE'
-                        self.stateShowPrompt()
-                
+                    self.text_ctrl.write("\x01\x1b[1;36m\x02 "+self.pager_lines[self.pager_index]+'\n')
+                    
+                for line in self.pager_lines[self.pager_index+1:self.pager_index+9]:
+                    self.text_ctrl.write("\x01\x1b[1;36m\x02 "+line+'\n')
+                self.pager_index += 10
+                self.pager_nb_lines -= 10
+                self.text_ctrl.write("--- Push Enter to continue or 'Q' to quit---")
+                self.pager_do_remove = True
+                self.pager_state = 'WAITING'
+                return
+            else:
+                #print >>sys.__stdout__,"PAGER processing last lines"
+                if self.pager_nb_lines > 0:
+                    if self.pager_index > 0:
+                        self.text_ctrl.write(">\x01\x1b[1;36m\x02"+self.pager_lines[self.pager_index]+'\n')
+                    else:
+                        self.text_ctrl.write("\x01\x1b[1;36m\x02 "+self.pager_lines[self.pager_index]+'\n')
+                            
+                    self.pager_index += 1
+                    self.pager_nb_lines -= 1
+                if self.pager_nb_lines > 0:
+                    for line in self.pager_lines[self.pager_index:]:
+                        self.text_ctrl.write("\x01\x1b[1;36m\x02 "+line+'\n')
+                        self.pager_nb_lines = 0
+                self.pager_state = 'DONE'
+                self.stateShowPrompt()
+            
     #------------------------ Key Handler ------------------------------------
     def keyPress(self, event):
         '''
@@ -740,11 +791,11 @@ class IPShellWidget(wx.Panel):
             
         if self.cur_state == 'IDLE':
             if event.KeyCode == wx.WXK_UP:
-                history = self.IP.historyBack()
+                history = self.IP.history_back()
                 self.text_ctrl.writeHistory(history)
                 return
             if event.KeyCode == wx.WXK_DOWN:
-                history = self.IP.historyForward()
+                history = self.IP.history_forward()
                 self.text_ctrl.writeHistory(history)
                 return
             if event.KeyCode == wx.WXK_TAB:
@@ -791,7 +842,20 @@ class IPShellWidget(wx.Panel):
         self.updateOptionTracker('background_color',
                                  self.options['background_color']['value'])
         self.text_ctrl.SetFocus()
-    
+
+    def evtCheckOptionThreading(self, event):
+        if event.IsChecked():
+            self.options['threading']['value']='True'
+            self.IP.set_threading(True)
+            self.cout.write = self.text_ctrl.asyncWrite
+        else:
+            self.options['threading']['value']='False'
+            self.IP.set_threading(False)
+            self.cout.write = self.text_ctrl.write
+        self.updateOptionTracker('threading',
+                                 self.options['threading']['value'])
+        self.text_ctrl.SetFocus()
+        
     def getOptions(self):
         return self.options
         
@@ -802,7 +866,13 @@ class IPShellWidget(wx.Panel):
             self.options[key]['checkbox'].SetValue(self.options[key][value])
             self.options[key]['setfunc'](value)
         
-        
+        if self.options['threading']['value']=='True':
+            self.IP.set_threading(True)
+            self.cout.write = self.text_ctrl.asyncWrite
+        else:
+            self.IP.set_threading(False)
+            self.cout.write = self.text_ctrl.write
+            
     #------------------------ Hook Section -----------------------------------
     def updateOptionTracker(self,name,value):
         '''
@@ -870,5 +940,3 @@ if __name__ == '__main__':
     shell = frame.shell
 
     app.MainLoop()
-
-
