@@ -5,32 +5,28 @@ IPython -- An enhanced Interactive Python
 Requires Python 2.1 or better.
 
 This file contains the main make_IPython() starter function.
-
-$Id: ipmaker.py 2930 2008-01-11 07:03:11Z vivainio $"""
+"""
 
 #*****************************************************************************
-#       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
+#       Copyright (C) 2008-2009 The IPython Development Team
+#       Copyright (C) 2001-2007 Fernando Perez. <fperez@colorado.edu>
 #
 #  Distributed under the terms of the BSD License.  The full license is in
 #  the file COPYING, distributed as part of this software.
 #*****************************************************************************
 
-from IPython import Release
-__author__  = '%s <%s>' % Release.authors['Fernando']
-__license__ = Release.license
-__version__ = Release.version
-
 try:
     credits._Printer__data = """
     Python: %s
 
-    IPython: Fernando Perez, Janko Hauser, Nathan Gray, and many users.
+    IPython: The IPython Development Team.
     See http://ipython.scipy.org for more information.""" \
     % credits._Printer__data
 
     copyright._Printer__data += """
 
-    Copyright (c) 2001-2004 Fernando Perez, Janko Hauser, Nathan Gray.
+    Copyright (c) 2008-2009 The IPython Development Team.
+    Copyright (c) 2001-2007 Fernando Perez, Janko Hauser, Nathan Gray.
     All Rights Reserved."""
 except NameError:
     # Can happen if ipython was started with 'python -S', so that site.py is
@@ -51,6 +47,7 @@ from pprint import pprint,pformat
 
 # Our own
 from IPython import DPyGetOpt
+from IPython import Release
 from IPython.ipstruct import Struct
 from IPython.OutputTrap import OutputTrap
 from IPython.ConfigLoader import ConfigLoader
@@ -58,9 +55,9 @@ from IPython.iplib import InteractiveShell
 from IPython.usage import cmd_line_usage,interactive_usage
 from IPython.genutils import *
 
-def force_import(modname):
-    if modname in sys.modules:
-        print "reload",modname
+def force_import(modname,force_reload=False):
+    if modname in sys.modules and force_reload:
+        info("reloading: %s" % modname)
         reload(sys.modules[modname])
     else:
         __import__(modname)
@@ -108,8 +105,6 @@ def make_IPython(argv=None,user_ns=None,user_global_ns=None,debug=1,
         IP.user_ns['help'] = _Helper()
     except ImportError:
         warn('help() not available - check site.py')
-    IP.user_config_ns = {}
-
 
     if DEVDEBUG:
         # For developer debugging only (global flag)
@@ -121,7 +116,7 @@ def make_IPython(argv=None,user_ns=None,user_global_ns=None,debug=1,
                          'for more information.\n'
                          % (sys.version.split('\n')[0],),
                          "IPython %s -- An enhanced Interactive Python."
-                         % (__version__,),
+                         % (Release.version,),
 """\
 ?         -> Introduction and overview of IPython's features.
 %quickref -> Quick reference.
@@ -131,20 +126,15 @@ object?   -> Details about 'object'. ?object also works, ?? prints more.
 
     IP.usage = interactive_usage
 
-    # Platform-dependent suffix and directory names.  We use _ipython instead
-    # of .ipython under win32 b/c there's software that breaks with .named
-    # directories on that platform.
+    # Platform-dependent suffix.
     if os.name == 'posix':
         rc_suffix = ''
-        ipdir_def = '.ipython'
     else:
         rc_suffix = '.ini'
-        ipdir_def = '_ipython'
 
     # default directory for configuration
-    ipythondir_def = os.path.abspath(os.environ.get('IPYTHONDIR',
-                                 os.path.join(IP.home_dir,ipdir_def)))
-
+    ipythondir_def = get_ipython_dir() 
+    
     sys.path.insert(0, '') # add . to sys.path. Fix from Prabhu Ramachandran
 
     # we need the directory where IPython itself is installed
@@ -230,6 +220,7 @@ object?   -> Details about 'object'. ?object also works, ?? prints more.
                       prompt_in2 = '   .\\D.: ',
                       prompt_out = 'Out[\\#]: ',
                       prompts_pad_left = 1,
+                      pydb = 0,
                       pylab = 0,
                       pylab_import_all = 1,
                       q4thread = 0,
@@ -334,7 +325,7 @@ object?   -> Details about 'object'. ?object also works, ?? prints more.
         sys.exit()
 
     if opts_all.Version:
-        print __version__
+        print Release.version
         sys.exit()
 
     if opts_all.magic_docstrings:
@@ -529,12 +520,6 @@ object?   -> Details about 'object'. ?object also works, ?? prints more.
 
     IP.pre_config_initialization()
     # configure readline
-    # Define the history file for saving commands in between sessions
-    if IP_rc.profile:
-        histfname = 'history-%s' % IP_rc.profile
-    else:
-        histfname = 'history'
-    IP.histfile = os.path.join(opts_all.ipythondir,histfname)
 
     # update exception handlers with rc file status
     otrap.trap_out()  # I don't want these messages ever.
@@ -640,23 +625,25 @@ object?   -> Details about 'object'. ?object also works, ?? prints more.
     except:
         IP.InteractiveTB()
         import_fail_info('ipy_system_conf')
-        
+
     # only import prof module if ipythonrc-PROF was not found
     if opts_all.profile and not profile_handled_by_legacy:
         profmodname = 'ipy_profile_' + opts_all.profile
         try:
-            
             force_import(profmodname)
         except:
             IP.InteractiveTB()
-            print "Error importing",profmodname,"- perhaps you should run %upgrade?"
+            print "Error importing",profmodname,\
+                  "- perhaps you should run %upgrade?"
             import_fail_info(profmodname)
+        else:
+            opts.profile = opts_all.profile
     else:
         force_import('ipy_profile_none')
+    # XXX - this is wrong: ipy_user_conf should not be loaded unconditionally,
+    # since the user could have specified a config file path by hand.
     try:
-            
         force_import('ipy_user_conf')
-        
     except:
         conf = opts_all.ipythondir + "/ipy_user_conf.py"
         IP.InteractiveTB()
@@ -665,6 +652,13 @@ object?   -> Details about 'object'. ?object also works, ?? prints more.
 
         import_fail_info("ipy_user_conf")
 
+    # Define the history file for saving commands in between sessions
+    try:
+        histfname = 'history-%s' % opts.profile
+    except AttributeError:
+        histfname = 'history'
+    IP.histfile = os.path.join(opts_all.ipythondir,histfname)
+
     # finally, push the argv to options again to ensure highest priority
     IP_rc.update(opts)
     
