@@ -15,10 +15,11 @@ if they need blocking clients or in `asyncclient.py` if they want asynchronous,
 deferred/Twisted using clients.
 """
 __docformat__ = "restructuredtext en"
-#-------------------------------------------------------------------------------
+#-----------------------------------------------------------------------------
 #  Copyright (C) 2008  The IPython Development Team
 #
 #  Distributed under the terms of the BSD License.  The full license is in
 #  the file COPYING, distributed as part of this software.
-#-------------------------------------------------------------------------------
-    
\ No newline at end of file
+#-----------------------------------------------------------------------------
+
+from IPython.kernel.error import TaskRejectError
\ No newline at end of file
