@@ -12,25 +12,36 @@ to use these functions in platform agnostic fashion.
 #  the file COPYING, distributed as part of this software.
 #*****************************************************************************
 
-from IPython import Release
-__author__  = '%s <%s>' % Release.authors['Ville']
-__license__ = Release.license
-
 import sys
 import os
 
-ignore_termtitle = False
+ignore_termtitle = True
 
 def _dummy_op(*a, **b):
     """ A no-op function """
 
+
 def _set_term_title_xterm(title):
     """ Change virtual terminal title in xterm-workalikes """
 
-    sys.stdout.write('\033]%d;%s\007' % (0,title))
+    sys.stdout.write('\033]0;%s\007' % title)
 
 
 if os.environ.get('TERM','') == 'xterm':
     set_term_title = _set_term_title_xterm
 else:
     set_term_title = _dummy_op
+
+
+def find_cmd(cmd):
+    """Find the full path to a command using which."""
+    return os.popen('which %s' % cmd).read().strip()
+
+
+def get_long_path_name(path):
+    """Dummy no-op."""
+    return path
+
+
+def term_clear():
+    os.system('clear')
