@@ -59,8 +59,7 @@ ColorSchemeTable class. Currently the following exist:
 You can implement other color schemes easily, the syntax is fairly
 self-explanatory. Please send back new schemes you develop to the author for
 possible inclusion in future releases.
-
-$Id: ultraTB.py 2908 2007-12-30 21:07:46Z vivainio $"""
+"""
 
 #*****************************************************************************
 #       Copyright (C) 2001 Nathaniel Gray <n8gray@caltech.edu>
@@ -70,11 +69,6 @@ $Id: ultraTB.py 2908 2007-12-30 21:07:46Z vivainio $"""
 #  the file COPYING, distributed as part of this software.
 #*****************************************************************************
 
-from IPython import Release
-__author__  = '%s <%s>\n%s <%s>' % (Release.authors['Nathan']+
-                                    Release.authors['Fernando'])
-__license__ = Release.license
-
 # Required modules
 import inspect
 import keyword
@@ -96,9 +90,9 @@ from inspect import getsourcefile, getfile, getmodule,\
 
 # IPython's own modules
 # Modified pdb which doesn't damage IPython's readline handling
-from IPython import Debugger, PyColorize
+from IPython import Debugger, PyColorize, ipapi
 from IPython.ipstruct import Struct
-from IPython.excolors import ExceptionColors
+from IPython.excolors import exception_colors
 from IPython.genutils import Term,uniq_stable,error,info
 
 # Globals
@@ -273,9 +267,10 @@ def _formatTracebackLines(lnum, index, lines, Colors, lvals=None,scheme=None):
 
     # This lets us get fully syntax-highlighted tracebacks.
     if scheme is None:
-        try:
-            scheme = __IPYTHON__.rc.colors
-        except:
+        ipinst = ipapi.get()
+        if ipinst is not None:
+            scheme = ipinst.IP.rc.colors
+        else:
             scheme = DEFAULT_SCHEME
     _line_format = _parser.format2
 
@@ -320,7 +315,7 @@ class TBTools:
         self.call_pdb = call_pdb
 
         # Create color table
-        self.color_scheme_table = ExceptionColors 
+        self.color_scheme_table = exception_colors()
 
         self.set_colors(color_scheme)
         self.old_scheme = color_scheme  # save initial value for toggles
@@ -445,7 +440,8 @@ class ListTB(TBTools):
         
         Also lifted nearly verbatim from traceback.py
         """
-        
+
+        have_filedata = False
         Colors = self.Colors
         list = []
         try:
@@ -459,8 +455,9 @@ class ListTB(TBTools):
                 try:
                     msg, (filename, lineno, offset, line) = value
                 except:
-                    pass
+                    have_filedata = False
                 else:
+                    have_filedata = True
                     #print 'filename is',filename  # dbg
                     if not filename: filename = "<string>"
                     list.append('%s  File %s"%s"%s, line %s%d%s\n' % \
@@ -490,6 +487,14 @@ class ListTB(TBTools):
                                               Colors.Normal, s))
             else:
                 list.append('%s\n' % str(stype))
+
+        # vds:>>
+        if have_filedata:
+            ipinst = ipapi.get()
+            if ipinst is not None:
+                ipinst.IP.hooks.synchronize_with_editor(filename, lineno, 0)
+        # vds:<<
+
         return list
 
     def _some_str(self, value):
@@ -801,6 +806,17 @@ class VerboseTB(TBTools):
             for name in names:
                 value = text_repr(getattr(evalue, name))
                 exception.append('\n%s%s = %s' % (indent, name, value))
+
+        # vds: >>
+        if records:
+             filepath, lnum = records[-1][1:3]
+             #print "file:", str(file), "linenb", str(lnum) # dbg
+             filepath = os.path.abspath(filepath)
+             ipinst = ipapi.get()
+             if ipinst is not None:
+                 ipinst.IP.hooks.synchronize_with_editor(filepath, lnum, 0)
+        # vds: <<
+                
         # return all our info assembled as a single string
         return '%s\n\n%s\n%s' % (head,'\n'.join(frames),''.join(exception[0]) )
 
